use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Anwar
Christian
DROLSKY
DROLSKY's
DateTime
Dave
Format
Hansen
Ilmari
Mannsåker
Measham
Mohammad
POSIX
PayPal
Rick
Rolsky
Rolsky's
STRPTIME
Strptime
Types
autarch
chansen
drolsky
errmsg
formatter
gregoa
gregor
herrmann
ilmari
key
lib
mohammad
rickm
strf
strp
strptime
yasutake
